@@ -1,7 +1,9 @@
 /*
- * /home/ms/files/source/libsidtune/RCS/MUS.cpp,v
+ * This file is part of libsidplayfp, a SID player engine.
  *
- * Sidplayer and Stereo Sidplayer format support.
+ * Copyright 2011-2013 Leandro Nini <drfiemost@users.sourceforge.net>
+ * Copyright 2007-2010 Antti Lankila
+ * Copyright 2000-2001 Simon White
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -15,42 +17,34 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-#include <string.h>
+#include "MUS.h"
 
-#ifdef HAVE_CONFIG_H
-#  include "config.h"
-#endif
+#include <string.h>
+#include <memory>
 
 #include "SidTuneCfg.h"
-#include "sidplayfp/SidTune.h"
+#include "SidTuneInfoImpl.h"
 #include "sidplayfp/sidendian.h"
+#include "sidplayfp/sidmemory.h"
 
-#ifdef HAVE_EXCEPTIONS
-#   include <new>
-#endif
+const char TXT_FORMAT_MUS[]        = "C64 Sidplayer format (MUS)";
+const char TXT_FORMAT_STR[]        = "C64 Stereo Sidplayer format (MUS+STR)";
 
-static const char _sidtune_txt_invalid[] = "ERROR: File contains invalid data";
-static const char _sidtune_txt_format_mus[] = "C64 Sidplayer format (MUS)";
-static const char _sidtune_txt_format_str[] = "C64 Stereo Sidplayer format (MUS+STR)";
-static const char _sidtune_txt_notEnoughMemory[] = "ERROR: Not enough free memory";
-static const char _sidtune_txt_sizeExceeded[] = "ERROR: Total file size too large";
+const char ERR_INVALID[]           = "ERROR: File contains invalid data";
+const char ERR_2ND_INVALID[]       = "ERROR: 2nd file contains invalid data";
+const char ERR_NOT_ENOUGH_MEMORY[] = "ERROR: Not enough free memory";
+const char ERR_SIZE_EXCEEDED[]     = "ERROR: Total file size too large";
 
 static const uint_least16_t SIDTUNE_MUS_HLT_CMD = 0x14F;
 
-static const uint_least16_t SIDTUNE_MUS_DATA_ADDR = 0x0900;
+static const uint_least16_t SIDTUNE_MUS_DATA_ADDR  = 0x0900;
 static const uint_least16_t SIDTUNE_SID1_BASE_ADDR = 0xd400;
 static const uint_least16_t SIDTUNE_SID2_BASE_ADDR = 0xd500;
 
-SidTune::LoadStatus SidTune::MUS_fileSupport(Buffer_sidtt<const uint_least8_t>& musBuf,
-                                             Buffer_sidtt<const uint_least8_t>& strBuf)
-{
-    return MUS_load (musBuf, strBuf, true);
-}
-
-bool SidTune::MUS_detect(const void* buffer, const uint_least32_t bufLen,
+bool MUS::detect(const uint_least8_t* buffer, uint_least32_t bufLen,
                          uint_least32_t& voice3Index)
 {
     SmartPtr_sidtt<const uint8_t> spMus((const uint8_t*)buffer,bufLen);
@@ -68,23 +62,40 @@ bool SidTune::MUS_detect(const void* buffer, const uint_least32_t bufLen,
             && spMus);
 }
 
-void SidTune::MUS_setPlayerAddress()
+void MUS::setPlayerAddress()
 {
-    if (info.sidChipBase2 == 0)
+    if (info->m_sidChipBase2 == 0)
     {
         // Player #1.
-        info.initAddr = 0xec60;
-        info.playAddr = 0xec80;
+        info->m_initAddr = 0xec60;
+        info->m_playAddr = 0xec80;
     }
     else
     {
         // Player #1 + #2.
-        info.initAddr = 0xfc90;
-        info.playAddr = 0xfc96;
+        info->m_initAddr = 0xfc90;
+        info->m_playAddr = 0xfc96;
     }
 }
 
-static const uint8_t _sidtune_sidplayer1[] =
+void MUS::acceptSidTune(const char* dataFileName, const char* infoFileName,
+                            Buffer_sidtt<const uint_least8_t>& buf, bool isSlashedFileName)
+{
+    setPlayerAddress();
+    SidTuneBase::acceptSidTune(dataFileName, infoFileName, buf, isSlashedFileName);
+}
+
+bool MUS::placeSidTuneInC64mem(sidmemory* mem)
+{
+    if (SidTuneBase::placeSidTuneInC64mem(mem))
+    {
+        installPlayer(mem);
+        return true;
+    }
+    return false;
+}
+
+static const uint8_t sidplayer1[] =
 {
     0x00, 0xe0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -290,7 +301,7 @@ static const uint8_t _sidtune_sidplayer1[] =
     0x60
 };
 
-static const uint8_t _sidtune_sidplayer2[] =
+static const uint8_t sidplayer2[] =
 {
     0x00, 0xf0, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
     0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
@@ -496,30 +507,28 @@ static const uint8_t _sidtune_sidplayer2[] =
     0x60, 0x00, 0x20, 0x60, 0xec, 0x4c, 0x60, 0xfc, 0x20, 0x80, 0xec, 0x4c, 0x80, 0xfc
 };
 
-bool SidTune::MUS_mergeParts(Buffer_sidtt<const uint_least8_t>& musBuf,
+bool MUS::mergeParts(Buffer_sidtt<const uint_least8_t>& musBuf,
                              Buffer_sidtt<const uint_least8_t>& strBuf)
 {
     Buffer_sidtt<uint8_t> mergeBuf;
 
-    uint_least32_t mergeLen = musBuf.len()+strBuf.len();
+    const uint_least32_t mergeLen = musBuf.len()+strBuf.len();
 
     // Sanity check. I do not trust those MUS/STR files around.
-    uint_least32_t freeSpace = endian_16(_sidtune_sidplayer1[1],_sidtune_sidplayer1[0])
+    const uint_least32_t freeSpace = endian_16(sidplayer1[1],sidplayer1[0])
                             - SIDTUNE_MUS_DATA_ADDR;
     if ( (musBuf.len()+strBuf.len()-4) > freeSpace)
     {
-        info.statusString = _sidtune_txt_sizeExceeded;
-        return false;
+        throw loadError(ERR_SIZE_EXCEEDED);
     }
 
-#ifdef HAVE_EXCEPTIONS
-    if ( !mergeBuf.assign(new(std::nothrow) uint8_t[mergeLen],mergeLen) )
-#else
-    if ( !mergeBuf.assign(new uint8_t[mergeLen],mergeLen) )
-#endif
+    try
     {
-        info.statusString = _sidtune_txt_notEnoughMemory;
-        return false;
+        mergeBuf.assign(new uint8_t[mergeLen], mergeLen);
+    }
+    catch (std::bad_alloc& e)
+    {
+        throw loadError(ERR_NOT_ENOUGH_MEMORY);
     }
 
     // Install MUS data #1 including load address.
@@ -529,7 +538,7 @@ bool SidTune::MUS_mergeParts(Buffer_sidtt<const uint_least8_t>& musBuf,
     memcpy((void*)mergeBuf.get(),musBuf.get(),musBuf.len());
 #endif
 
-    if ( !strBuf.isEmpty() && info.sidChipBase2!=0 )
+    if ( !strBuf.isEmpty() && info->m_sidChipBase2!=0 )
     {
         // Install MUS data #2 _NOT_ including load address.
 #ifndef SID_HAVE_BAD_COMPILER
@@ -545,107 +554,97 @@ bool SidTune::MUS_mergeParts(Buffer_sidtt<const uint_least8_t>& musBuf,
     return true;
 }
 
-void SidTune::MUS_installPlayer(uint_least8_t *c64buf)
+void MUS::installPlayer(sidmemory *mem)
 {
-    if (status && (c64buf != 0))
+    if (mem != 0)
     {
         // Install MUS player #1.
-        uint_least16_t dest = endian_16(_sidtune_sidplayer1[1],
-                                     _sidtune_sidplayer1[0]);
-        memcpy(c64buf+dest,_sidtune_sidplayer1+2,sizeof(_sidtune_sidplayer1)-2);
+        uint_least16_t dest = endian_16(sidplayer1[1], sidplayer1[0]);
+        mem->fillRam(dest, sidplayer1+2, sizeof(sidplayer1)-2);
         // Point player #1 to data #1.
-        c64buf[dest+0xc6e] = (SIDTUNE_MUS_DATA_ADDR+2)&0xFF;
-        c64buf[dest+0xc70] = (SIDTUNE_MUS_DATA_ADDR+2)>>8;
+        mem->writeMemByte(dest+0xc6e, (SIDTUNE_MUS_DATA_ADDR+2)&0xFF);
+        mem->writeMemByte(dest+0xc70, (SIDTUNE_MUS_DATA_ADDR+2)>>8);
 
-        if (info.sidChipBase2 != 0)
+        if (info->m_sidChipBase2 != 0)
         {
             // Install MUS player #2.
-            dest = endian_16(_sidtune_sidplayer2[1],
-                             _sidtune_sidplayer2[0]);
-            memcpy(c64buf+dest,_sidtune_sidplayer2+2,sizeof(_sidtune_sidplayer2)-2);
+            dest = endian_16(sidplayer2[1], sidplayer2[0]);
+            mem->fillRam(dest, sidplayer2+2, sizeof(sidplayer2)-2);
             // Point player #2 to data #2.
-            c64buf[dest+0xc6e] = (SIDTUNE_MUS_DATA_ADDR+musDataLen+2)&0xFF;
-            c64buf[dest+0xc70] = (SIDTUNE_MUS_DATA_ADDR+musDataLen+2)>>8;
+            mem->writeMemByte(dest+0xc6e, (SIDTUNE_MUS_DATA_ADDR+musDataLen+2)&0xFF);
+            mem->writeMemByte(dest+0xc70, (SIDTUNE_MUS_DATA_ADDR+musDataLen+2)>>8);
         }
     }
 }
 
-SidTune::LoadStatus SidTune::MUS_load (Buffer_sidtt<const uint_least8_t>& musBuf, bool init)
+SidTuneBase* MUS::load (Buffer_sidtt<const uint_least8_t>& musBuf, bool init)
 {
     Buffer_sidtt<const uint_least8_t> empty;
-    return MUS_load (musBuf, empty, init);
+    return load (musBuf, empty, 0, init);
 }
 
-SidTune::LoadStatus SidTune::MUS_load (Buffer_sidtt<const uint_least8_t>& musBuf,
+SidTuneBase* MUS::load (Buffer_sidtt<const uint_least8_t>& musBuf,
                                        Buffer_sidtt<const uint_least8_t>& strBuf,
+                                       uint_least32_t fileOffset,
                                        bool init)
 {
     uint_least32_t voice3Index;
-    SmartPtr_sidtt<const uint8_t> spPet(musBuf.get()+fileOffset,musBuf.len()-fileOffset);
-    if ( !MUS_detect(&spPet[0],spPet.tellLength(),voice3Index) )
-        return LOAD_NOT_MINE;
+    SmartPtr_sidtt<const uint8_t> spPet(musBuf.get()+fileOffset, musBuf.len()-fileOffset);
+    if ( !detect(&spPet[0], spPet.tellLength(), voice3Index) )
+        return 0;
 
+    std::auto_ptr<MUS> tune(new MUS());
+    tune->tryLoad(musBuf, strBuf, spPet, voice3Index, init);
+    tune->mergeParts(musBuf, strBuf);
+
+    return tune.release();
+}
+
+void MUS::tryLoad(Buffer_sidtt<const uint_least8_t>& musBuf,
+                                       Buffer_sidtt<const uint_least8_t>& strBuf,
+                                       SmartPtr_sidtt<const uint8_t> &spPet,
+                                       uint_least32_t voice3Index,
+                                       bool init)
+{
     if (init)
     {
-        info.songs = (info.startSong = 1);
-        info.musPlayer = true;
+        info->m_songs = (info->m_startSong = 1);
 
-        songSpeed[0]  = SIDTUNE_SPEED_CIA_1A;
-#ifdef SIDTUNE_PSID2NG
-        clockSpeed[0] = SIDTUNE_CLOCK_ANY;
-#endif
+        songSpeed[0]  = SidTuneInfo::SPEED_CIA_1A;
+        clockSpeed[0] = SidTuneInfo::CLOCK_ANY;
     }
 
     // Check setting compatibility for MUS playback
-    if ((info.compatibility != SIDTUNE_COMPATIBILITY_C64) ||
-        (info.relocStartPage != 0) || (info.relocPages != 0))
+    if ((info->m_compatibility != SidTuneInfo::COMPATIBILITY_C64) ||
+        (info->m_relocStartPage != 0) || (info->m_relocPages != 0))
     {
-        info.formatString = _sidtune_txt_invalid;
-        return LOAD_ERROR;
+        throw loadError(ERR_INVALID);
     }
 
     {   // All subtunes should be CIA
-        for (uint_least16_t i = 0; i < info.songs; i++)
+        for (uint_least16_t i = 0; i < info->m_songs; i++)
         {
-            if (songSpeed[i] != SIDTUNE_SPEED_CIA_1A)
+            if (songSpeed[i] != SidTuneInfo::SPEED_CIA_1A)
             {
-                info.formatString = _sidtune_txt_invalid;
-                return LOAD_ERROR;
+                throw loadError(ERR_INVALID);
             }
         }
     }
 
     musDataLen = musBuf.len();
-    info.loadAddr = SIDTUNE_MUS_DATA_ADDR;
-    info.sidChipBase1 = SIDTUNE_SID1_BASE_ADDR;
-
-    // No credits so extract them from the MUS files
-    bool credits = (infoString[0][0] | infoString[1][0] | infoString[2][0]) != 0;
+    info->m_loadAddr = SIDTUNE_MUS_DATA_ADDR;
+    info->m_sidChipBase1 = SIDTUNE_SID1_BASE_ADDR;
 
     // Voice3Index now is offset to text lines (uppercase Pet-strings).
     spPet += voice3Index;
 
-    // Already have credits just skip over the ones in the MUS
-    if (credits)
-    {
-        while (spPet[0])
-            convertPetsciiToAscii(spPet,0);
-    }
     // Extract credits
-    else
+    while (spPet[0])
     {
-        for (int line = info.numberOfInfoStrings = 0; spPet[0]; line =
-             ++info.numberOfInfoStrings)
-        {
-            if (line < 10)
-            {
-                convertPetsciiToAscii(spPet,infoString[line]);
-                info.infoString[line] = infoString[line];
-            }
-            else
-                convertPetsciiToAscii(spPet,0);
-        }
+        PetsciiToAscii converter;
+        info->m_commentString.push_back(converter.convert(spPet));
     }
+
     spPet++;
 
     // If we appear to have additional data at the end, check is it's
@@ -653,8 +652,8 @@ SidTune::LoadStatus SidTune::MUS_load (Buffer_sidtt<const uint_least8_t>& musBuf
     bool stereo = false;
     if ( !strBuf.isEmpty() )
     {
-        if ( !MUS_detect(strBuf.get(),strBuf.len(),voice3Index) )
-            return LOAD_ERROR;
+        if ( !detect(strBuf.get(), strBuf.len(), voice3Index) )
+            throw loadError(ERR_2ND_INVALID);
         spPet.setBuffer (strBuf.get(),strBuf.len());
         stereo = true;
     }
@@ -663,7 +662,7 @@ SidTune::LoadStatus SidTune::MUS_load (Buffer_sidtt<const uint_least8_t>& musBuf
         if ( spPet.good() )
         {
             uint_least16_t pos = (uint_least16_t) spPet.tellPos();
-            if ( MUS_detect(&spPet[0],spPet.tellLength()-pos,voice3Index) )
+            if ( detect(&spPet[0],spPet.tellLength()-pos,voice3Index) )
             {
                 musDataLen = pos;
                 stereo = true;
@@ -675,59 +674,33 @@ SidTune::LoadStatus SidTune::MUS_load (Buffer_sidtt<const uint_least8_t>& musBuf
     {   // Voice3Index now is offset to text lines (uppercase Pet-strings).
         spPet += voice3Index;
 
-        // Already have credits just skip over the ones in the MUS
-        if (credits)
-        {
-            while (spPet[0])
-                convertPetsciiToAscii(spPet,0);
-        }
         // Extract credits
-        else
+        while (spPet[0])
         {
-            for (int line = info.numberOfInfoStrings; spPet[0]; line =
-                ++info.numberOfInfoStrings)
-            {
-                if (line < 10)
-                {
-                    convertPetsciiToAscii(spPet,infoString[line]);
-                    info.infoString[line] = infoString[line];
-                }
-                else
-                    convertPetsciiToAscii(spPet,0);
-            }
+            PetsciiToAscii converter;
+            info->m_commentString.push_back(converter.convert(spPet));
         }
 
-        info.sidChipBase2 = SIDTUNE_SID2_BASE_ADDR;
-        info.formatString = _sidtune_txt_format_str;
+        info->m_sidChipBase2 = SIDTUNE_SID2_BASE_ADDR;
+        info->m_formatString = TXT_FORMAT_STR;
     }
     else
     {
-        info.sidChipBase2 = 0;
-        info.formatString = _sidtune_txt_format_mus;
+        info->m_sidChipBase2 = 0;
+        info->m_formatString = TXT_FORMAT_MUS;
     }
-    MUS_setPlayerAddress();
 
-    if (!credits)
-    {   // Remove trailing empty lines.
-        const int lines = info.numberOfInfoStrings;
-        {
-            for ( int line = lines-1; line >= 0; line-- )
-            {
-                if (strlen(info.infoString[line]) == 0)
-                    --info.numberOfInfoStrings;
-                else
-                    break;
-            }
-        }
+    setPlayerAddress();
 
-        // Three strings are assumed to be credits in
-        // the format title, author and released, which
-        // these are not
-        if (info.numberOfInfoStrings == 3)
+    // Remove trailing empty lines.
+    const int lines = info->m_commentString.size();
+    {
+        for ( int line = lines-1; line >= 0; line-- )
         {
-            info.infoString[3] = &infoString[3][0];
-            info.numberOfInfoStrings++;
+            if (info->m_commentString[line].length() == 0)
+                info->m_commentString.pop_back();
+            else
+                break;
         }
     }
-    return LOAD_OK;
 }
