@@ -1,5 +1,5 @@
 /*
- *  Copyright (C) 2010 Leandro Nini
+ *  Copyright (C) 2010-2011 Leandro Nini
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,46 +16,40 @@
  *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-#ifndef _INIPARSER_
-#define _INIPARSER_
+#ifndef INIPARSER_H
+#define INIPARSER_H
 
 #include <string>
 #include <iostream>
 #include <fstream>
 #include <map>
 
-class iniParser {
+class iniParser
+{
+private:
+    typedef std::map<std::string, std::string> keys_t;
 
-public:
-	static long parseLong(const char* str);
-
-	static double parseDouble(const char* str);
+    class emptyPair {};
 
 private:
-	typedef std::map<std::string, std::string> keys_t;
+    std::map<std::string, keys_t> sections;
 
-	std::map<std::string, keys_t> sections;
+    std::map<std::string, keys_t>::const_iterator curSection;
 
-	class emptyPair {};
+private:
+    std::string parseSection(const char *buffer);
 
-	std::string parseSection(const char* buffer);
-	std::pair<std::string, std::string> parseKey(const char* buffer);
-
-	std::map<std::string, keys_t>::const_iterator curSection;
+    std::pair<std::string, std::string> parseKey(const char *buffer);
 
 public:
-	class parseError : public std::exception {};
+    iniParser() {}
+    ~iniParser() {}
 
-public:
-	iniParser() {};
-	~iniParser() {};
+    bool open(const char *fName);
+    void close();
 
-	bool open(const char* fName);
-	void close();
-
-	bool setSection(const char* section);
-	const char* getValue(const char* key);
+    bool setSection(const char *section);
+    const char *getValue(const char *key);
 };
 
-#endif
-
+#endif // INIPARSER_H
